package view;

import javafx.beans.value.ObservableValue;
import javafx.collections.*;
import javafx.fxml.FXML;
import javafx.geometry.Pos;
import javafx.scene.control.*;
import javafx.scene.layout.*;
import javafx.scene.text.Font;
import javafx.util.Callback;
import model.*;
import java.io.IOException;


public class MainPage {

    @FXML
    private Button FirstButton;

    @FXML
    private VBox vb1;

    @FXML
    public void initialize() throws IOException {
        CapteurVirtuel cV1 = new CapteurVirtuel("CaptVirt1");
        CPUTemp cpuTemp = new CPUTemp("CPU Captor");

        Captor c1 = new Captor("c1");
        Captor c2 = new Captor("c2");
        Captor c3 = new Captor("c3");

        ListView<CapteurVirtuel> listview = new ListView<>();

        VBox FirstCaptorCPU = new VBox();
        FirstCaptorCPU.setMinSize(565,470);
        // FirstCaptorCPU.setBorder(new Border(new BorderStroke(Color.BLUE, BorderStrokeStyle.SOLID, CornerRadii.EMPTY, new BorderWidths(3))));

        javafx.scene.image.Image CPUImage = new javafx.scene.image.Image("https://media.istockphoto.com/id/1204740322/fr/photo/cpu.jpg?s=612x612&w=0&k=20&c=cF154ASgimXysleBPyYsFMI9_GKkkTpDQ0kZTJnm_K0=");
        javafx.scene.image.ImageView FirstCaptorCPUImage = new javafx.scene.image.ImageView(CPUImage);

        Label FirstCaptorCPULabel = new Label("Temperature");
        FirstCaptorCPULabel.setFont(new Font("Arial", 30));
        FirstCaptorCPULabel.setMaxWidth(Double.MAX_VALUE);
        FirstCaptorCPULabel.setAlignment(Pos.CENTER);

        FirstCaptorCPU.getChildren().add(FirstCaptorCPUImage);
        FirstCaptorCPU.getChildren().add(FirstCaptorCPULabel);



        ObservableList<CPUTemp> CPUTemperature = FXCollections.observableArrayList(
                captor-> new ObservableValue[]{
                        captor.getTemperatureProperty()
                }
        );

        CPUTemperature.addListener((ListChangeListener.Change<? extends CPUTemp> c) ->{
            while (c.next()){
                if (c.wasUpdated()){
                    FirstCaptorCPULabel.textProperty().set("Temperature du CPU : " +  cpuTemp.getTemperature() + "°C");
                }
            }
        });

        ObservableList<CapteurVirtuel> captors = FXCollections.observableArrayList(
                captor->new ObservableValue[]{
                        captor.getTempMoyenneProperty()
                }
        );

        captors.addListener((ListChangeListener.Change<? extends CapteurVirtuel> c) ->{
            while (c.next()){
                if (c.wasUpdated()){
                    listview.refresh();
                }
            }
        });

        c1.startThread();
        c2.startThread();
        c3.startThread();

        cV1.addToLesCapteurs(c1);
        cV1.addToLesCapteurs(c2);
        cV1.addToLesCapteurs(c3);

        cV1.startThread();

        cpuTemp.startThread();

        captors.add(cV1);

        listview.setCellFactory(new Callback<>() {

            @Override
            public ListCell<CapteurVirtuel> call(ListView<CapteurVirtuel> captorListView) {
                return new ListCell<>() {
                    @Override
                    protected void updateItem(CapteurVirtuel value, boolean empty) {
                        String text = "";
                        super.updateItem(value, empty);
                        if (value != null) {
                            text = value.getTempMoyenne().toString();
                        }
                        setText(text);
                        String style = null;
                        if (!empty) {
                            assert value != null;
                            if (value.isNegative()) {
                                style = "-fx-text-fill : blue";
                            }
                        }
                        if (!empty && value.isHot()) {
                            style = "-fx-text-fill : red";
                        }
                        setStyle(style);
                    }
                };
            }
        });

        listview.setItems(captors);
        FirstButton.textProperty().set(cpuTemp.getName());
        FirstCaptorCPULabel.textProperty().set("Temperature du CPU : " +  cpuTemp.getTemperature() + "°C");
        FirstButton.setOnAction(e -> {
            vb1.getChildren().clear();
            vb1.getChildren().add(FirstCaptorCPU);
        });
    }


}
